import discord
from cpl_core.configuration import ConfigurationABC
from cpl_discord.events import OnGuildChannelUpdateABC
from discord import CategoryChannel

from bot_core.logging.message_logger import MessageLogger
from bot_data.abc.realm_repository_abc import RealmRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.realm_moderator import RealmModerator
from modules.realms.realm_utils import RealmUtils


class RealmOnChannelUpdateEvent(OnGuildChannelUpdateABC):
    def __init__(
        self,
        config: ConfigurationABC,
        logger: MessageLogger,
        servers: ServerRepositoryABC,
        users: UserRepositoryABC,
        realms: RealmRepositoryABC,
        realm_utils: RealmUtils,
    ):
        OnGuildChannelUpdateABC.__init__(self)

        self._config = config
        self._logger = logger
        self._servers = servers
        self._users = users
        self._realms = realms
        self._realm_utils = realm_utils

    async def on_guild_channel_update(self, before: discord.abc.GuildChannel, after: discord.abc.GuildChannel):
        self._logger.debug(__name__, f"Module {type(self)} started")

        realm = self._realms.find_realm_by_name(before.name)
        if realm is None:
            return

        if before.name != after.name:
            realm.name = after.name
            self._realms.update_realm(realm)
            role = self._realm_utils.find_role_by_name(after.guild, before.name)
            if role is not None:
                await role.edit(name=after.name)

        if before.overwrites != after.overwrites:
            role = self._realm_utils.get_role_by_name(after.guild, after.name)
            if role not in after.overwrites:
                await after.set_permissions(
                    role,
                    view_channel=True,
                    connect=True,
                    manage_channels=True,
                    manage_permissions=True,
                    create_instant_invite=True,
                )

            mods = self._realms.get_realm_moderators_by_realm_id(realm.id)
            mod_users = mods.select(lambda x: x.user)
            for mod in mods:
                if mod.user not in after.overwrites:
                    self._realms.delete_realm_moderator(mod)

            server = self._servers.get_server_by_discord_id(after.guild.id)
            for mod in after.overwrites:
                if isinstance(mod, discord.Role):
                    continue

                if mod not in mod_users:
                    user = self._users.find_user_by_discord_id_and_server_id(mod.id, server.id)
                    self._realms.add_realm_moderator(RealmModerator(realm, user))
                    await self._realm_utils.assign_moderator(after, mod)

        self._logger.debug(__name__, f"Module {type(self)} stopped")
